@@ -1,8 +1,10 @@
 # SHAP visualization functions for XGBoost,
-# wrapped up functions for
+# wrapped functions for
 # summary plot, dependence plot, force plot, and interaction effect plot
 # Further explained on my research blog: https://liuyanguu.github.io/post/2019/07/18/visualization-of-shap-for-xgboost/
-# Please cite http://doi.org/10.5281/zenodo.3334713
+# The package was applied in paper http://doi.org/10.5281/zenodo.3334713
+#
+# Please cite if this is useful.
 #
 
 
@@ -11,18 +13,25 @@ if(getRversion() >= "2.15.1")  {
                            "x_feature", "mean_value",
                            "int_value", "color_value",
                            "new_labels","labels_within_package",
-                           "group", "rest_variables", "clusterid", "id", "BIAS"))
+                           "..var_cat",
+  "group", "rest_variables", "clusterid", "ID", "sorted_id", "BIAS"))
   }
 
 
 # data preparation functions ----------------------------------------------
+
 #' return SHAP contribution from xgboost model
 #'
-#' \code{shap.values} returns from xgboost model a list of 1.the matrix of shap score and
-#' 2. the ranked variable vector by each variable's mean absolute SHAP value
+#' The \code{shap.values} returns a list of three objects from xgboost model: 1.
+#' a dataset (data.table) of SHAP scores. It has the same dimension as the
+#' X_train); 2. the ranked variable vector by each variable's mean absolute SHAP
+#' value, it ranks the predictors by their importance in the model; and 3. The
+#' BIAS, which is like an intercept. The rowsum of SHAP values including the
+#' BIAS would equal to the predicted value (y_hat).
 #'
 #' @param xgb_model a xgboost model object
-#' @param X_train the dataset of predictors used for the xgboost model
+#' @param X_train the dataset of predictors (independent variables) used for the
+#'   xgboost model
 #'
 #' @import data.table
 #' @import xgboost
@@ -30,7 +39,8 @@ if(getRversion() >= "2.15.1")  {
 #'
 #' @export shap.values
 #'
-#' @return a list of three elements, the SHAP values as data.table, ranked mean|SHAP|, BIAS
+#' @return a list of three elements: the SHAP values as data.table, ranked
+#'   mean|SHAP|, and BIAS
 #'
 #' @example R/example/example_fit_summary.R
 #'
@@ -55,34 +65,51 @@ shap.values <- function(xgb_model,
 
 #' prep SHAP values into long format for plotting
 #'
-#' @param xgb_model a xgboost model object
-#' @param shap_contrib optional to supply SHAP values dataset, default to NULL
-#' @param X_train the dataset of predictors used for the xgboost model
-#' if not NULL, will be taken as SHAP values,
+#' Produce a dataset of 6 columns: ID of each observation, variable name, SHAP
+#' value, variable values (feature value), deviation of the feature value for
+#' each observation (for coloring the point), and the mean SHAP values for each
+#' variable. You can view this example dataset included in the package:
+#' \code{\link{shap_long_iris}}
+#'
+#' The ID variable is added for each observation in the `shap_contrib` dataset
+#' for better tracking, it is created as `1:nrow(shap_contrib)` before melting
+#' `shap_contrib` into long format.
+#'
+#' @param xgb_model a xgboost model object, will derive the SHAP values from it
+#' @param shap_contrib optional to directly supply a SHAP values dataset. If
+#'   supplied, it will overwrite the `xgb_model` if `xgb_model` is also supplied
+#' @param X_train the dataset of predictors used for the xgboost model, it
+#'   provides feature values to the plot, must be supplied
 #' @param top_n to choose top_n variables ranked by mean|SHAP| if needed
+#' @param var_cat if supplied, will provide long format data, grouped by this
+#'   categorical variable
 #'
 #' @import data.table
 #' @export shap.prep
 #'
-#' @return a long-format data.table, named as `shap_long` in other functions
+#' @return a long format data.table, named as `shap_long`
 #'
 #' @example R/example/example_fit_summary.R
+#' @example R/example/example_categorical.R
 #'
 shap.prep <- function(xgb_model = NULL,
-                      shap_contrib = NULL, # optional to supply SHAP values
+                      shap_contrib = NULL, # optional to directly supply SHAP values
                       X_train,
-                      top_n = NULL){
+                      top_n = NULL,
+                      var_cat = NULL
+                      ){
+  if (is.null(xgb_model) & is.null(shap_contrib)) stop("Please provide either `xgb_model` or `shap_contrib`")
   if (!is.null(shap_contrib)){
     if(paste0(dim(shap_contrib), collapse = " ") != paste0(dim(X_train), collapse = " ")) stop("supply correct shap_contrib, remove BIAS column.\n")
   }
+
   # prep long-data
   shap <- if (is.null(shap_contrib)) shap.values(xgb_model, X_train) else list(
-    shap_score = shap_contrib,
-    mean_shap_score = colMeans(abs(shap_contrib))[order(colMeans(abs(shap_contrib)), decreasing = T)]
-  )
-
+      shap_score = shap_contrib,
+      mean_shap_score = colMeans(abs(shap_contrib))[order(colMeans(abs(shap_contrib)), decreasing = TRUE)]
+    )
   std1 <- function(x){
-    return ((x - min(x, na.rm = T))/(max(x, na.rm = T) - min(x, na.rm = T)))
+    return ((x - min(x, na.rm = TRUE))/(max(x, na.rm = TRUE) - min(x, na.rm = TRUE)))
   }
 
   # choose top n features
@@ -93,22 +120,41 @@ shap.prep <- function(xgb_model = NULL,
     top_n <- dim(X_train)[2]
   }
 
-  # descending order
-  shap_score_sub <- setDT(shap$shap_score)[, names(shap$mean_shap_score)[1:top_n], with = F]
-  shap_score_long <- melt.data.table(shap_score_sub, measure.vars = colnames(shap_score_sub))
-
-  # feature values: the values in the original dataset
-  # dayint is int, will throw a warning here
+  # arrange variables in descending order, thus the summary plot could be
+  # plotted accordingly.
+  shap_score_sub <- setDT(shap$shap_score)[, names(shap$mean_shap_score)[1:top_n], with = FALSE]
+  shap_score_sub[, ID:= .I]
+  # fv: feature values: the values in the original dataset
+  # fv_sub: subset of feature values
+  # since dayint is int, the package example will throw a warning here
   fv_sub <- as.data.table(X_train)[, names(shap$mean_shap_score)[1:top_n], with = F]
-  # standardize feature values
-  fv_sub_long <- melt.data.table(fv_sub, measure.vars = colnames(fv_sub))
 
-  fv_sub_long[, stdfvalue := std1(value), by = "variable"]
+  if(is.null(var_cat)){
+    # shap_score_sub contains the sample ID
+    shap_score_long <- melt.data.table(shap_score_sub, measure.vars = colnames(fv_sub))
+    vars_wanted <- colnames(fv_sub)
+
+  } else if (var_cat%in%colnames(fv_sub)) {
+    # exclude var_cat as it is used as a categorical group
+    shap_score_long <- melt.data.table(shap_score_sub[,-..var_cat], measure.vars = colnames(fv_sub)[!colnames(fv_sub) %in% c(var_cat, "ID")])
+    vars_wanted <- colnames(fv_sub)[!colnames(fv_sub) %in% var_cat]
+  } else {
+    stop("Please provide a correct var_cat variable, a categorical variable that
+         exists in the dataset.")
+  }
+  # standardize feature values
+  fv_sub_long <- melt.data.table(fv_sub, measure.vars = vars_wanted, value.name = "rfvalue")
+  fv_sub_long[, stdfvalue := std1(rfvalue), by = "variable"]
   # SHAP value: value
   # raw feature value: rfvalue;
   # standarized: stdfvalue
-  names(fv_sub_long) <- c("variable", "rfvalue", "stdfvalue" )
-  shap_long2 <- cbind(shap_score_long, fv_sub_long[,c('rfvalue','stdfvalue')])
+  if(is.null(var_cat)){
+    shap_long2 <- cbind(shap_score_long, fv_sub_long[,c('rfvalue','stdfvalue')])
+  } else {
+    shap_long2 <- cbind(shap_score_long, fv_sub_long[,c('rfvalue','stdfvalue', var_cat), with = FALSE])
+  }
+  # mean_value: mean abs SHAP values by variable, used as the label by
+  # `geom_text` in the summary plot
   shap_long2[, mean_value := mean(abs(value)), by = variable]
   setkey(shap_long2, variable)
   return(shap_long2)
@@ -117,8 +163,8 @@ shap.prep <- function(xgb_model = NULL,
 
 #' prepare the interaction SHAP values from predict.xgb.Booster
 #'
-#' This function just runs \code{shap_int <- predict(xgb_mod, as.matrix(X_train), predinteraction = TRUE)}, may not be necessary,
-#' maybe just use xgboost::predict.xgb.Booster directly,
+#' This function just runs \code{shap_int <- predict(xgb_mod, as.matrix(X_train), predinteraction = TRUE)}, thus it may not be necessary.
+#' Read more about the xgboost predict function at `xgboost::predict.xgb.Booster`.
 #'
 #' @param xgb_model a xgboost model object
 #' @param X_train the dataset of predictors used for the xgboost model
@@ -135,25 +181,32 @@ shap.prep.interaction <- function(xgb_model, X_train){
 
 # SHAP summary plot ----------------------------------------------------------
 
-#' SHAP summary plot core function using the long-format SHAP values
+#' SHAP summary plot core function using the long format SHAP values
 #'
-#' The summary plot (sina plot) uses a long-format data of SHAP values. The long-format data
-#' could be obtained from either xgboost model or a SHAP matrix using \code{\link{shap.values}}.
-#' If you want to start with xgbmodel and data_X, use \code{\link{shap.plot.summary.wrap1}}.
-#' If you want to use self-derived SHAP matrix, use \code{\link{shap.plot.summary.wrap2}}.
-#' If a global list named **new_labels** is provided (\code{!is.null(new_labels}),
-#' the plots will use that list to replace default labels \code{\link{labels_within_package}}.
-#' @param data_long a long format data of SHAP values from \code{\link{shap.prep}}
-#' @param x_bound in case need to limit x_axis_limit
-#' @param dilute a number or logical, dafault to TRUE, will plot \code{nrow(data_long)/dilute} data.
-#' for example, if dilute = 5 will plot 1/5 of the data.
-#' If dilute = TRUE or a number, we will plot at most half points per feature, so the plot won't be too slow.
-#' If you put dilute too high, at least 10 points per feature would be kept.
-#' If the dataset is even smaller than that, will just plot all the data.
-#' @param scientific  show the mean|SHAP| in scientific format or not
-#' default to F, label format is 0.000,
-#' If true, label format is 0.0E-0,
-#' @param my_format supply your own number format if you really want to do so
+#' The summary plot (a sina plot) uses a long format data of SHAP values. The
+#' SHAP values could be obtained from either an xgboost model or a SHAP value
+#' matrix using \code{\link{shap.values}}. So this summary plot function
+#' normally follows the long format dataset obtained using `shap.values`. If you
+#' want to start with a xgboost model and data_X, use
+#' \code{\link{shap.plot.summary.wrap1}}. If you want to use a self-derived
+#' dataset of SHAP values, use \code{\link{shap.plot.summary.wrap2}}. If a list
+#' named **new_labels** is provided in the environment (`new_labels` is
+#' pre-loaded by the package as \code{NULL}), the plots will use that list to
+#' label the variables, here is an example of such a list (the default labels):
+#' \code{\link{labels_within_package}}.
+#'
+#' @param data_long a long format data of SHAP values from
+#'   \code{\link{shap.prep}}
+#' @param x_bound to set horizontal axis limit in the plot
+#' @param dilute a number or logical, aim to make the test plot for large amount
+#'   of data faster. If dilute = 5 will plot 1/5 of the data. If dilute = TRUE
+#'   or a number, will plot at most half points per feature, so the plotting
+#'   won't be too slow. If you put dilute too high, at least 10 points per
+#'   feature would be kept. If the dataset is too small after dilution, will
+#'   just plot all the data
+#' @param scientific  show the mean|SHAP| in scientific format. If TRUE, label
+#'   format is 0.0E-0, default to FALSE, whose format is 0.000
+#' @param my_format supply your own number format if you really want
 #'
 #' @import ggplot2
 #' @importFrom ggforce geom_sina
@@ -164,7 +217,8 @@ shap.prep.interaction <- function(xgb_model, X_train){
 #'
 shap.plot.summary <- function(data_long,
                               x_bound = NULL,
-                              dilute = FALSE, scientific = FALSE,
+                              dilute = FALSE,
+                              scientific = FALSE,
                               my_format = NULL){
 
   if (scientific){label_format = "%.1e"} else {label_format = "%.3f"}
@@ -183,8 +237,9 @@ shap.plot.summary <- function(data_long,
   }
 
   x_bound <- if (is.null(x_bound)) max(abs(data_long$value))*1.1 else as.numeric(abs(x_bound))
-  plot1 <- ggplot(data = data_long)+
+  plot1 <- ggplot(data = data_long) +
     coord_flip(ylim = c(-x_bound, x_bound)) +
+    geom_hline(yintercept = 0) + # the y-axis beneath
     # sina plot:
     ggforce::geom_sina(aes(x = variable, y = value, color = stdfvalue),
               method = "counts", maxwidth = 0.7, alpha = 0.7) +
@@ -207,7 +262,6 @@ shap.plot.summary <- function(data_long,
           legend.title=element_text(size=10),
           legend.text=element_text(size=8),
           axis.title.x= element_text(size = 10)) +
-    geom_hline(yintercept = 0) + # the vertical line
     # reverse the order of features, from high to low
     # also relabel the feature using `label.feature`
     scale_x_discrete(limits = rev(levels(data_long$variable)),
@@ -219,8 +273,6 @@ shap.plot.summary <- function(data_long,
 #' A wrapped function to make summary plot from xgb model object and predictors
 #'
 #' wraps up function \code{\link{shap.prep}} and \code{\link{shap.plot.summary}}
-#' If a global list named **new_labels** is provided (\code{!is.null(new_labels}),
-#' the plots will use that list to replace default labels \code{\link{labels_within_package}}.
 #' @param model the xgboost model
 #' @param X the dataset of predictors used for the xgboost model
 #' @param top_n how many predictors you want to show in the plot (ranked)
@@ -243,10 +295,7 @@ shap.plot.summary.wrap1 <- function(model, X, top_n, dilute = FALSE){
 #' Sometimes the SHAP matrix is returned from cross-validation.
 #' This function wraps up function \code{\link{shap.prep}} and \code{\link{shap.plot.summary}}.
 #'
-#' If a global list named **new_labels** is provided (\code{!is.null(new_labels}),
-#' the plots will use that list to replace default labels \code{\link{labels_within_package}}.
-#'
-#' @param shap_score the SHAP values dataset, could be obtained by \code{shap.prep}.
+#' @param shap_score the SHAP values dataset, could be obtained by \code{shap.prep}
 #' @param X the dataset of predictors used for the xgboost model
 #' @param top_n how many predictors you want to show in the plot (ranked)
 #' @inheritParams shap.plot.summary
@@ -267,15 +316,17 @@ shap.plot.summary.wrap2 <- function(shap_score, X, top_n, dilute = FALSE){
 
 #' helper function to modify labels for features under plotting
 #'
-#' If a list is created in the environment named **new_labels** (\code{!is.null(new_labels}),
-#' the plots will use that list to replace default list of labels \code{\link{labels_within_package}}.
+#' If a list is created in the global environment named **new_labels**
+#' (\code{!is.null(new_labels}), the plots will use that list to replace default
+#' list of labels \code{\link{labels_within_package}}.
 #'
 #' @param x variable names
 #'
 #' @return a character, e.g. "date", "Time Trend", etc.
 #'
 label.feature <- function(x){
-  labs = labels_within_package # a saved list of some feature names that I am using
+  # a saved list of some feature names that I am using
+  labs <- SHAPforxgboost::labels_within_package
   # but if you supply your own `new_labels`, it will print your feature names
   # must provide a list.
   if (!is.null(new_labels)) {
@@ -283,7 +334,8 @@ label.feature <- function(x){
       message("new_labels should be a list, for example,`list(var0 = 'VariableA')`.\n")
       }  else {
       message("Plot will use user-defined labels.\n")
-      labs = new_labels}
+      labs = new_labels
+      }
   }
   out <- rep(NA, length(x))
   for (i in 1:length(x)){
@@ -324,52 +376,58 @@ plot.label <- function(plot1, show_feature){
 #' SHAP dependence plot and interaction plot, optional to be colored by a
 #' selected feature
 #'
-#' This function makes the simple dependence plot with SHAP values on the y axis, optional to add color
-#' by another feature, optional to use a different y variable for SHAP values
-#' Not colored if \code{color_feature} is not supplied.
-#' If \code{data_int} (the SHAP interaction values dataset) is supplied, it will plot
-#' the interaction effect between \code{y} and \code{x} on the y axis.
+#' This function by default makes a simple dependence plot with feature values
+#' on the x-axis and SHAP values on the y-axis, optional to color by another
+#' feature. It is optional to use a different variable for SHAP values on the
+#' y-axis, and color the points by the feature value of a designated variable.
+#' Not colored if \code{color_feature} is not supplied. If \code{data_int} (the
+#' SHAP interaction values dataset) is supplied, it will plot the interaction
+#' effect between \code{y} and \code{x} on the y-axis. Dependence plot is easy
+#' to make if you have the SHAP values dataset from \code{predict.xgb.Booster}.
+#' It is not necessary to start with the long format data, but since that is
+#' used for the summary plot, we just continue to use it here.
 #'
-#' Dependence plot is very easy to make if you have the SHAP values dataset from \code{predict.xgb.Booster}
-#' It is not necessary to start with the long-format data, but since I used that
-#' for the summary plot, I just continue to use the long dataset
 #' @param data_long the long format SHAP values from \code{\link{shap.prep}}
-#' @param data_int the 3-dimention SHAP interaction values array.
-#' if `data_int` is supplied, y axis will plot the interaction values of y (vs. x)
-#' @param dilute a number or logical, dafault to TRUE,
-#' will plot \code{nrow(data_long)/dilute} data.
-#' For example, if dilute = 5 will plot 20% of the data.
-#' As long as dilute != FALSE, will plot at most half the data.
 #'
-#' from \code{predict.xgb.Booster} or \code{\link{shap.prep.interaction}}.
-#' @param x which feature to show on x axis, it will plot the feature value.
-#' @param y which shap values to show on y axis, it will plot the SHAP value of that feature.
-#' y is default to x, if y is not provided, just plot the SHAP values of x on the y axis
-#' @param color_feature which feature value to use for coloring, color by the feature value.
-#' @param smooth optional to add _loess_ smooth line, default to TRUE.
-#' @param size0 point size, default to 1 of nobs<1000, 0.4 if nobs>1000.
-#' @param add_hist whether to add histogram using \code{ggMarginal}, default to TRUE.
-#' But notice the plot after adding histogram it is \code{ggExtraPlot} object, cannot
-#' add geom to that anymore. If wish to add more ggplot layers, turn the histogram off
+#' @param x which feature to show on x-axis, it will plot the feature value
+#' @param y which shap values to show on y-axis, it will plot the SHAP value of
+#'   that feature. y is default to x, if y is not provided, just plot the SHAP
+#'   values of x on the y-axis
+#' @param color_feature which feature value to use for coloring, color by the
+#'   feature value
+#' @param data_int the 3-dimention SHAP interaction values array. if `data_int`
+#'   is supplied, y-axis will plot the interaction values of y (vs. x).
+#'   `data_int` is obtained from either \code{predict.xgb.Booster} or
+#'   \code{\link{shap.prep.interaction}}
+#' @param dilute a number or logical, dafault to TRUE, will plot
+#'   \code{nrow(data_long)/dilute} data. For example, if dilute = 5 will plot
+#'   20% of the data. As long as dilute != FALSE, will plot at most half the
+#'   data
+#' @param smooth optional to add a _loess_ smooth line, default to TRUE.
+#' @param size0 point size, default to 1 of nobs<1000, 0.4 if nobs>1000
+#' @param add_hist whether to add histogram using \code{ggMarginal}, default to
+#'   TRUE. But notice the plot after adding histogram is a `ggExtraPlot` object
+#'   instead of `ggplot2` so cannot add `geom` to that anymore. Turn the
+#'   histogram off if you wish to add more `ggplot2` geoms
 #'
 #' @export shap.plot.dependence
 #'
-#' @return returns a ggplot2 object, based on which you could add more geom layers.
+#' @return be default a `ggplot2` object, based on which you could add more geom
+#'   layers.
 #'
 #' @example R/example/example_dependence_plot.R
 #'
-shap.plot.dependence <- function(data_long,
-                                       x,
-                                       y = NULL,
-                                       color_feature = NULL,
-                                       data_int = NULL,  # if supply, will plot SHAP interaction values
-                                       dilute = FALSE,
-
-                                       smooth = TRUE,
-                                       size0 = NULL,
-                                       add_hist = FALSE
-                                       )
-  {
+shap.plot.dependence <- function(
+  data_long,
+  x,
+  y = NULL,
+  color_feature = NULL,
+  data_int = NULL,  # if supply, will plot SHAP
+  dilute = FALSE,
+  smooth = TRUE,
+  size0 = NULL,
+  add_hist = FALSE
+  ){
   if (is.null(y)) y <- x
   data0 <- data_long[variable == y,.(variable, value)] # the shap value to plot for dependence plot
   data0$x_feature <- data_long[variable == x, rfvalue]
@@ -396,7 +454,6 @@ shap.plot.dependence <- function(data_long,
                       y = if (is.null(data_int)) value else int_value,
                       color = if (!is.null(color_feature)) color_value else NULL))+
     geom_point(size = size0, alpha = if(nrow(data0)<1000L) 1 else 0.6)+
-    # a loess smoothing line:
     labs(y = if (is.null(data_int)) paste0("SHAP value for ", label.feature(y)) else
       paste0("SHAP interaction values for\n", label.feature(x), " and ", label.feature(y)) ,
          x = label.feature(x),
@@ -408,6 +465,7 @@ shap.plot.dependence <- function(data_long,
     theme(legend.position="bottom",
           legend.title=element_text(size=10),
           legend.text=element_text(size=8))
+    # a loess smoothing line:
   if(smooth){
     plot1 <- plot1 + geom_smooth(method = 'loess', color = 'red', size = 0.4, se = F)
   }
@@ -423,32 +481,43 @@ shap.plot.dependence <- function(data_long,
 
 #' Prepare data for SHAP force plot (stack plot)
 #'
-#' Make force plot for \code{top_n} features, option to randomly
-#' plot certain portion of the data in case the dataset is large.
+#' Make force plot for \code{top_n} features, optional to randomly plot certain
+#' portion of the data in case the dataset is large.
 #'
-#' @param shap_contrib shap_contrib is the SHAP value data returned from predict.xgb.booster
-#' @param data_percent what percent of data to plot (to speed up), in the range of (0,1]
+#' @param shap_contrib shap_contrib is the SHAP value data returned from
+#'   predict.xgb.booster, here an ID variable is added for each observation in
+#'   the `shap_contrib` dataset for better tracking, it is created in the
+#'   begining as `1:nrow(shap_contrib)`. The ID matches the output from
+#'   \code{\link{shap.prep}}
+#' @param data_percent what percent of data to plot (to speed up the testing
+#'   plot). The accepted input range is (0,1], if observations left is too few,
+#'   there will be an error from the clustering function
 #' @param top_n integer, optional to show only top_n features, combine the rest
-#' @param cluster_method default to ward.D
-#' @param n_groups a integer, how many groups to plot in \code{\link{shap.plot.force_plot_bygroup}}
+#' @param cluster_method default to ward.D, please refer to `stats::hclust` for
+#'   details
+#' @param n_groups a integer, how many groups to plot in
+#'   \code{\link{shap.plot.force_plot_bygroup}}
 #'
 #' @export shap.prep.stack.data
 #' @return a dataset for stack plot
 #'
 #' @example R/example/example_force_plot.R
 #'
-shap.prep.stack.data <- function(shap_contrib,
-                            top_n = NULL,
-                            data_percent = 1,
-                            cluster_method = 'ward.D',
-                            n_groups = 10L){
+shap.prep.stack.data <- function(
+  shap_contrib,
+  top_n = NULL,
+  data_percent = 1,
+  cluster_method = 'ward.D',
+  n_groups = 10L
+  ){
   ranked_col <- names(colMeans(abs(shap_contrib))[order(colMeans(abs(shap_contrib)), decreasing = T)])
   shap_contrib2 <- setDT(shap_contrib)[,ranked_col, with = F]
-
+  shap_contrib2[, ID:= .I]
   # sample `data_percent` of the data, making the plot faster
   set.seed(1234)
   if (data_percent>1|data_percent<=0) data_percent <- 1
   shap_contrib2 <- shap_contrib2[sample(.N, .N * data_percent)]
+  # setkey(shap_contrib2, ID)
 
   # select columns into `shapobs`, default to all columns
   if(is.null(top_n)) top_n <- length(ranked_col)
@@ -459,7 +528,7 @@ shap.prep.stack.data <- function(shap_contrib,
     bottom_col <- ranked_col[-(1:top_n)]
     shap_contrib2[, rest_variables:= rowSums(.SD), .SDcol = bottom_col]
     # dataset with desired variables for plot
-    shapobs <- shap_contrib2[, c(top_ranked_col, "rest_variables"), with = F]
+    shapobs <- shap_contrib2[, c("ID", top_ranked_col, "rest_variables"), with = F]
     message("The SHAP values of the Rest ", length(ranked_col) - top_n, " features were summed into variable 'rest_variables'.\n")
   } else {
     shapobs <- shap_contrib2
@@ -472,20 +541,21 @@ shap.prep.stack.data <- function(shap_contrib,
   shapobs$group <- cutree(clusters, n_groups)
   shapobs <- shapobs[, clusterid:= clusters$order][rank(clusterid),]
   shapobs[,clusterid:=NULL]
-  shapobs[,id:=.I]
+  shapobs[,sorted_id:=.I] # this is the id that sort all the obs
   return(shapobs)
 }
 
 
 #' make the SHAP force plot
 #'
-#' The force/stack plot, optional to zoom in at certain x or certain cluster.
-#'
+#' The force/stack plot, optional to zoom in at certain x-axis location or zoom
+#' in a specific cluster of observations.
 #'
 #' @param shapobs The dataset obtained by \code{shap.prep.stack.data}.
 #' @param id the id variable.
-#' @param zoom_in_location where to zoom in, default at place of 60 percent of the data.
-#' @param y_parent_limit  set y axis limits.
+#' @param zoom_in_location where to zoom in, default at place of 60 percent of
+#'   the data.
+#' @param y_parent_limit  set y-axis limits.
 #' @param y_zoomin_limit  \code{c(a,b)} to limit the y-axis in zoom-in.
 #' @param zoom_in default to TRUE, zoom in by \code{ggforce::facet_zoom}.
 #' @param zoom_in_group optional to zoom in certain cluster.
@@ -496,15 +566,18 @@ shap.prep.stack.data <- function(shap_contrib,
 #'
 #' @example R/example/example_force_plot.R
 #'
-shap.plot.force_plot <- function(shapobs, id = 'id',
-                            zoom_in_location = NULL,
-                            y_parent_limit = NULL,
-                            y_zoomin_limit = NULL,   # c(a,b) to limit the y-axis in zoom-in
-                            zoom_in = TRUE,  # default zoom in to zoom_in_location
-                            zoom_in_group = NULL # if is set (!NULL), zoom-in to the defined cluster
+shap.plot.force_plot <- function(
+  shapobs,
+  id = 'sorted_id',
+  zoom_in_location = NULL,
+  y_parent_limit = NULL,
+  y_zoomin_limit = NULL,# c(a,b) to limit the y-axis in zoom-in
+  zoom_in = TRUE,  # default zoom in to zoom_in_location
+  zoom_in_group = NULL # if is set (!NULL), zoom-in to the defined cluster
 ){
   # optional: location to zoom in certain part of the plot
-  shapobs_long <- melt.data.table(shapobs, id.vars = c(id,"group"))
+  shapobs_long <- melt.data.table(shapobs,
+       measure.vars = colnames(shapobs)[!colnames(shapobs)%in%c(id, "group", "ID")])
   # display.brewer.pal("Paired")
   p <- ggplot(shapobs_long, aes_string(x = id, y = "value" , fill = "variable")) +
     geom_col(width =1, alpha = 0.9) +
@@ -557,11 +630,12 @@ shap.plot.force_plot <- function(shapobs, id = 'id',
 
 #' make the stack plot, optional to zoom in at certain x or certain cluster
 #'
-#' A collective display of zoom in plot: one plot of every group of clustered observations.
+#' A collective display of zoom-in plots: one plot for every group of the
+#' clustered observations.
 #'
 #' @param shapobs The dataset obtained by \code{shap.prep.stack.data}.
 #' @param id the id variable.
-#' @param y_parent_limit  set y axis limits.
+#' @param y_parent_limit  set y-axis limits.
 #'
 #' @importFrom RColorBrewer brewer.pal
 #' @importFrom ggforce facet_zoom
@@ -569,11 +643,14 @@ shap.plot.force_plot <- function(shapobs, id = 'id',
 #'
 #' @example R/example/example_force_plot.R
 #'
-shap.plot.force_plot_bygroup <- function(shapobs, id = 'id',
-                                     y_parent_limit = NULL
+shap.plot.force_plot_bygroup <- function(
+  shapobs,
+  id = 'sorted_id',
+  y_parent_limit = NULL
 ){
   # optional: location to zoom in certain part of the plot
-  shapobs_long <- melt.data.table(shapobs, id.vars = c(id,"group"))
+  shapobs_long <- melt.data.table(shapobs,
+    measure.vars = colnames(shapobs)[!colnames(shapobs)%in%c(id, "group", "ID")])
   p <- ggplot(shapobs_long, aes_string(x = id, y = "value" , fill = "variable")) +
     geom_col(width =1, alpha = 0.9) +
     facet_wrap(~ group, scales = "free", ncol = 2) +
